(in-package :ha)

;;;=====================
;;; FILE WRITING
;;;=====================

; Writes the content of a list of integers
(defmethod aux-savefile-intlist ((l list) (file pathname))
  (with-open-file (out file :direction :output 
                       :if-does-not-exist :create :if-exists :supersede)
    (loop for item in l do
          (format out "~d~A" item #\Linefeed)))
)

(defun split-by-one-space (string)
    "FROM: http://cl-cookbook.sourceforge.net/strings.html
Returns a list of substrings of string
divided by ONE space each.
Note: Two consecutive spaces will be seen as
if there were an empty string between them."
    (loop for i = 0 then (1+ j)
          as j = (position #\Space string :start i)
          collect (subseq string i j)
          while j))

(read-from-string (first (split-by-one-space "4 maj")))
(defmethod analysis-from-textfile ((source pathname))
  (with-open-file (in source)
    (list
     ; Reads the root labels
     (let ((line (read-line in nil 'eof)))
       (read-from-string line))
     ; Reads the chord types (vocabulary)
     (let ((line (read-line in nil 'eof)))
       (read-from-string line))
     ; Reads the segments
     (loop for line = (read-line in nil 'eof)
           until (eq line 'eof)
           collect
           (read-from-string line)
           #|(let ((tokens (split-by-one-space line)))
             (list (read-from-string (first tokens)) ; root
                   (second tokens) ; type
                   (read-from-string (third tokens)) ; id
                   (read-from-string (fourth tokens)) ; onset
                   (read-from-string (fifth tokens)) ; offset
                   '()))|#
           #|(multiple-value-bind (root pos) (read-from-string line t nil :start 0)
             (multiple-value-bind (type line2) (string-until-char line " ")
               (multiple-value-bind (id pos) (read-from-string line2 t nil :start 0)
                 (multiple-value-bind (onset pos) (read-from-string line2 t nil :start pos)
                   (multiple-value-bind (offset pos) (read-from-string line2 t nil :start pos)
                     (multiple-value-bind (points pos) (read-from-string line2 t nil :start pos)
                       (list root type id onset offset points)))))))|#
           ))))

(defmethod chordseq-from-textfile ((source pathname))
  (with-open-file (in source)
    (list
     (om::data-from-line (read-line in nil 'eof))
     (om::data-from-line (read-line in nil 'eof))
     (om::data-from-line (read-line in nil 'oef)))
    ))

;;;==========
;;; GRAPHICS
;;;==========

;------------
; Methods for obtaining the coordinates of the objects in pixels
; OM staff methods defined in /code/projects/02-musicproject/editor/scoreeditor/scoretools.lisp
;------------
(defun get-staff-top-px (view)
  (* (om::staff-size view) (om::score-top-margin view)))

(defun get-staff-height-px (view)
  (om::get-system-size (om::staff-sys view) (om::staff-size view)))

(defun get-one-staff-height-px (view)
  (round (* (om::staff-size view) (om::staff-h (car (om::staff-list (om::staff-sys view)))))))

(defun get-staff-line-height-px (view)
  (let* ((staff (car (om::staff-list (om::staff-sys view))))
         (staff-h (round (* (om::staff-size view) (om::staff-h staff)))))
    (round (/ staff-h (om::numlines staff)))))

(defun get-onset-in-px (onset view)
  (+ (om::get-key-space view)
     (om::ms2pixel onset (/ (om::staff-size view) 4) (om::staff-zoom view))))
;------------

(defun get-font-size (view)
  (om::staff-size view))

(defun get-note-size (view)
  (round (/ (om::staff-size view) 4)))

