(in-package :ha)

;;;================
;;; SERVER METHODS
;;;================

; CONFIGURATION OPTIONS FOR THE OSC ANALYSIS SERVER
(defparameter *analysis-server-host* "127.0.0.1") 
;(defparameter *analysis-server-host* "129.102.72.137") 
(defparameter *analysis-server-port* 3000)
;(defparameter *analysis-server-classpath* (concatenate 'string (namestring *ha-java-directory*) "*"))
(defparameter *analysis-server-classpath* "/Users/cperez/Documents/workspace/imtonalanalysis2-osc-server/bin:/Users/cperez/Documents/workspace/imtonalanalysis2-osc-server/lib/*:/Users/cperez/Documents/workspace/imcore2/bin:/Users/cperez/Documents/workspace/imcore2/lib/*:/Users/cperez/Documents/workspace/imtonalanalysis2/bin:/Users/cperez/Documents/workspace/imtonalanalysis2/lib/*")
(defparameter *analysis-server-command* (concatenate 'string "java -classpath \"" *analysis-server-classpath* "\" es.ua.dlsi.im2.tonalanalysis.oscserver.OSCServer &"))

; CONFIGURATION OPTIONS FOR RECEIVING OSC MESSAGES FROM THE SERVER
(defparameter *analysis-in-port* 7001)
(defparameter *print-server-msg* t)

(defmethod! osc-stop-server ()
  (if (analysis-server-msg 5 nil "/ok" "/stop")
      t
    nil))

(defmethod! osc-check-server ()
  (if (analysis-server-msg 2 nil "/ok" "/check")
      t
    nil))

(defmethod! osc-start-server ()
  (if (not (osc-check-server))
      (progn
        (sys::call-system *analysis-server-command* :wait nil)
        (sleep 2)
        (if (osc-check-server)
            (progn
              (print "Analysis server started")
              t)
          (progn
            (print "Failed launching analysis server")
            nil))
        )
    (progn
      (print "Analysis server already started")
      nil))
  )

(defmethod! osc-restart-server()
  (osc-stop-server)
  (osc-start-server))

;(osc-start-server)
;(osc-check-server)
;(osc-stop-server)

(defun analysis-server-msg (wait retry repmess &rest data)
  (let ((smsg data))
    (when *print-server-msg* (print (format nil "SENT: ~A" smsg))) 
    (if wait
        (let* ((reply nil)
               (message nil)
               (wait-time (if (numberp wait) wait 10))
               (count 0)
               ;(*message-win* (init-message-win))
               (answer-process (om-start-osc-server *analysis-in-port* "localhost"  
                                                    #'(lambda (msg host) 
                                                        (let* ((rmsg (om-decode-msg-or-bundle msg))
                                                               (msgcontents (if (listp (cadr rmsg)) (cadr rmsg) rmsg)))
                                                          (when *print-server-msg* (print (format nil "RECEIVED: ~A" msgcontents)))
                             ;(when (and (equal wait 'progress-bar)
                             ;           (string-equal (car msgcontents) "/busy")
                             ;           (> (length msgcontents) 1))
                             ;  (setf count 0)  ;; reinit time-out
                             ;  (update-progress-bar (cdr msgcontents))
                             ;  )
                                                          (when (and (not reply)
                                                                     (or (not repmess) 
                                                                         (and (stringp (car msgcontents))
                                                                              (or (string-equal repmess (car msgcontents))
                                                                                  (string-equal "/error" (car msgcontents))))))
                                                            (setf reply t)
                                                            (setf message msgcontents)
                                                            )
                                                          nil)))))    
          ; (when (equal wait 'progress-bar)
          ;   (init-progress-bar))
          (om-send-osc-message *analysis-server-port* *analysis-server-host* smsg)
          (when (equal wait 'progress-bar)
            (show-message-win "SERVER is working. Please wait..." :size (om-make-point 400 110)))
          
          (loop ; for count = 0 then (+ count 1)
           while (not reply) do
           (sleep 0.2)
           (setf count (+ count 1))
           (when (> count (/ wait-time 0.2))
             (if retry
                 (if (om-y-or-n-dialog (string+ "Server not responding to " (car smsg) ". Wait " (number-to-string wait-time) "s. more?"))
                     (setf count 0)
                   (setf reply t))
               (setf reply t))))
          
          (om-stop-osc-server answer-process)
          (if (string-equal "/error" (car message))
              (progn
                (om-message-dialog (format nil "ERROR at ~A ~A.~%=> ~s" (car smsg) (cadr smsg) (print (cadr message))))
                nil)
          #|(when (equal wait 'progress-bar)
            (hide-message-win))|#
            message))
      (oa::om-send-osc-message *analysis-server-port* *analysis-server-host* smsg)
      )
    ))


#|(defun update-progress-bar (msgs)
  (change-message-win (format nil "~A ~%~D %" 
                              (or (nth 1 msgs) "...") 
                              (round (* 100 (nth 0 msgs))))))
|#

(defun osc-send-receive (&rest args)
  (let ((response (apply #'analysis-server-msg 5 t "/ok" args)))
    (if response
        (second response)
      nil)))

(defun osc-new-analysis (path)
  (osc-send-receive "/analysis/harmonic" "new" path))

(defun osc-analyze (id)
  (let ((analysis-path (osc-send-receive "/analysis/harmonic" "analyze" id)))
    (if (and analysis-path (probe-file (pathname analysis-path)))
        (list id
              (print (analysis-from-textfile (pathname analysis-path))))
      (osc-harmonic-analysis-error))))

(defmethod osc-harmonic-analysis-error ()
  (om-beep-msg "Error in analysis !")
  '(nil ()))

(defmethod! osc-harmonic-analysis ((self HARMONIC-ANALYSIS) (song om::chord-seq))
  (let* ((temp-file-name (om::save-as-midi song (tmpfile "in-file.mid")))
         (id (osc-new-analysis temp-file-name)))
    (if id
        (osc-analyze id)
      (osc-harmonic-analysis-error))))

(defmethod! osc-harmonic-analysis-with-onsets ((self HARMONIC-ANALYSIS) (song om::chord-seq) (onsets list))
  (let* ((temp-file-name (om::save-as-midi song (tmpfile "in-file.mid")))
         (tmp-onsets-file (save-data onsets (tmpfile "onsets.txt")))
         (id (osc-new-analysis temp-file-name)))
    (if id
        (if (osc-send-receive "/analysis/harmonic" "set" id "input-onsets" (namestring tmp-onsets-file))
            (osc-analyze id)
          (osc-harmonic-analysis-error))
      (osc-harmonic-analysis-error))))

(defun osc-harmonic-analysis-change-label (an-id seg-id root type)
  (let ((analysis-path (osc-send-receive "/analysis/harmonic" "set" an-id "chord-label" seg-id root type)))
    (if (and analysis-path (probe-file (pathname analysis-path)))
        (print (analysis-from-textfile (pathname analysis-path)))
      nil)))

#|(defmethod! osc-harmonic-analysis-with-onsets ((self HARMONIC-ANALYSIS) (song om::chord-seq) (onsets list))
  (let ((tmp-onsets-file (tmpfile "onsets.txt")))
    (save-data onsets tmp-onsets-file)
    (let* ((tmp-midi-file (om::save-as-midi song (tmpfile "in-file.mid")))
           (analysis-rep (cadr (analysis-server-msg 5 t "/ok" "/analysis/harmonic" tmp-midi-file "input-onsets" (namestring tmp-onsets-file)))))
      (if (and analysis-rep (probe-file (pathname analysis-rep)))
          (print (analysis-from-textfile (pathname analysis-rep)))
        (progn
          (om-beep-msg "Error in analysis !")
          ;;; TEMP
          '()))
      ))
)|#


;;; EXPERIMENTAL - DEPRECATED

(defmethod! osc-harmonic-analysis-as-chordseq ((song t))
  (let* ((temp-file-name (om::save-as-midi song (tmpfile "in-file.mid")))
         (analysis-rep (cadr (analysis-server-msg 5 t "/ok" "/analysis/harmonic" temp-file-name "output-chordseq"))))
          (if (and analysis-rep (probe-file (pathname analysis-rep)))
              (print (chordseq-from-textfile (pathname analysis-rep)))
            (progn
              (om-beep-msg "Error in analysis !")
              ;;; TEMP
              '()))
          )
  )
  
(defmethod! osc-tonal-analysis ((song om::voice))
  (let* ((temp-file-name (om::export-musicxml
                          song
                          '((G 2)) 2 (namestring (tmpfile "in-file.xml"))))
         (analysis-rep (cadr (analysis-server-msg 5 t "/ok" "/analysis/tonal" temp-file-name))))
          (if (and analysis-rep (probe-file (pathname analysis-rep)))
              (print (analysis-from-textfile (pathname analysis-rep)))
            (progn
              (om-beep-msg "Error in analysis !")
              ;;; TEMP
              '()))
          )
  )

;;; STARTUP ACTIONS

; Server will be closed with OpenMusic
(om-add-exit-cleanup-func 'osc-stop-server)

; The server is started when the library is loaded
(when *ha-autostart-server*
  (osc-start-server))
