(in-package :ha)

; CONFIGURATION OPTIONS FOR THE OSC ANALYSIS SERVER
(defparameter *analysis-server-host* "127.0.0.1") 
(defparameter *analysis-server-port* 3000)
(defparameter *analysis-server-classpath* (concatenate 'string (namestring *ha-java-directory*) "*"))
;(defparameter *analysis-server-classpath* "/Users/cperez/Documents/workspace/imtonalanalysis2-osc-server/bin:/Users/cperez/Documents/workspace/imtonalanalysis2-osc-server/lib/*:/Users/cperez/Documents/workspace/imcore2/bin:/Users/cperez/Documents/workspace/imcore2/lib/*:/Users/cperez/Documents/workspace/imtonalanalysis2/bin:/Users/cperez/Documents/workspace/imtonalanalysis2/lib/*")
(defparameter *analysis-server-command* (concatenate 'string "java -classpath \"" *analysis-server-classpath* "\" es.ua.dlsi.im2.tonalanalysis.oscserver.OSCServer &"))

; CONFIGURATION OPTIONS FOR RECEIVING OSC MESSAGES FROM THE SERVER
(defparameter *analysis-in-port* 7001)
(defparameter *print-server-msg* t)


; FUNCTION FOR COMMUNICATING WITH THE SERVER
(defun analysis-server-msg (wait retry repmess &rest data)
  (let ((smsg data))
    (when *print-server-msg* (print (format nil "SENT: ~A" smsg))) 
    (if wait
        (let* ((reply nil)
               (message nil)
               (wait-time (if (numberp wait) wait 10))
               (count 0)
               ;(*message-win* (init-message-win))
               (answer-process (om-start-osc-server *analysis-in-port* "localhost"  
                                                    #'(lambda (msg) 
                                                        (let* ((rmsg (om-decode-msg-or-bundle msg))
                                                               (msgcontents (if (listp (cadr rmsg)) (cadr rmsg) rmsg)))
                                                          (when *print-server-msg* (print (format nil "RECEIVED: ~A" msgcontents)))
                             ;(when (and (equal wait 'progress-bar)
                             ;           (string-equal (car msgcontents) "/busy")
                             ;           (> (length msgcontents) 1))
                             ;  (setf count 0)  ;; reinit time-out
                             ;  (update-progress-bar (cdr msgcontents))
                             ;  )
                                                          (when (and (not reply)
                                                                     (or (not repmess) 
                                                                         (and (stringp (car msgcontents))
                                                                              (or (string-equal repmess (car msgcontents))
                                                                                  (string-equal "/error" (car msgcontents))))))
                                                            (setf reply t)
                                                            (setf message msgcontents)
                                                            )
                                                          nil)))))    
          ; (when (equal wait 'progress-bar)
          ;   (init-progress-bar))
          (om-send-osc-message *analysis-server-port* *analysis-server-host* smsg)
          (when (equal wait 'progress-bar)
            (show-message-win "SERVER is working. Please wait..." :size (om-make-point 400 110)))
          
          (loop ; for count = 0 then (+ count 1)
           while (not reply) do
           (sleep 0.2)
           (setf count (+ count 1))
           (when (> count (/ wait-time 0.2))
             (if retry
                 (if (om-y-or-n-dialog (string+ "Server not responding to " (car smsg) ". Wait " (number-to-string wait-time) "s. more?"))
                     (setf count 0)
                   (setf reply t))
               (setf reply t))))
          
          (om-stop-osc-server answer-process)
          (if (string-equal "/error" (car message))
              (progn
                (om-message-dialog (format nil "ERROR at ~A ~A.~%=> ~s" (car smsg) (cadr smsg) (print (cadr message))))
                nil)
          #|(when (equal wait 'progress-bar)
            (hide-message-win))|#
            message))
      (oa::om-send-osc-message *analysis-server-port* *analysis-server-host* smsg)
      )
    ))


#|(defun update-progress-bar (msgs)
  (change-message-win (format nil "~A ~%~D %" 
                              (or (nth 1 msgs) "...") 
                              (round (* 100 (nth 0 msgs))))))
|#

; UTILITY FUNCTION FOR SENDING A MESSAGE AND RECEIVE THE ANSWER
(defun osc-send-receive (&rest args)
  (let ((response (apply #'analysis-server-msg 5 t "/ok" args)))
    (if response
        (second response)
      nil)))

;;;=============================
;;; SERVER MANAGEMENT FUNCTIONS
;;;=============================

(defun osc-stop-server ()
  (if (analysis-server-msg 5 nil "/ok" "/stop")
      t
    nil))

(defun osc-check-server ()
  (if (analysis-server-msg 2 nil "/ok" "/check")
      t
    nil))

(defun osc-start-server ()
  (if (not (osc-check-server))
      (progn
        (sys::call-system *analysis-server-command* :wait nil)
        (sleep 2)
        (if (osc-check-server)
            (progn
              (print "Analysis server started")
              t)
          (progn
            (print "Failed launching analysis server")
            nil))
        )
    (progn
      (print "Analysis server already started")
      nil))
  )

(defun osc-restart-server()
  (osc-stop-server)
  (osc-start-server))

; (osc-restart-server) 

;;;=================
;;; STARTUP ACTIONS
;;;=================

; Server will be closed with OpenMusic
(om-add-exit-cleanup-func 'osc-stop-server)

; The server is started when the library is loaded
(when *ha-autostart-server*
  (osc-start-server))
